program amr2mpeg
  !--------------------------------------------------------------------------
  ! Ce programme calcule la carte de densite projetee pour les
  ! variables hydro d'une simulation RAMSES. 
  ! amr2map F90 par R. Teyssier le 01/04/01.
  ! map2mpeg par Y. Rasera le 12/03/07
  !--------------------------------------------------------------------------
  implicit none
  integer::ndim,n,i,j,k,twotondim,ncoarse,type=0,domax=0
  integer::ivar,nvar,ncpu,ncpuh,lmax=0
  integer::nx,ny,nz,ilevel,idim,jdim,kdim,icell
  integer::nlevelmax,ilevel1,ngrid1
  integer::nlevelmaxs,nlevel,iout
  integer::ind,ipos,ngrida,ngridh,ilevela,ilevelh
  integer::ngridmax,nstep_coarse,icpu,ncpu_read
  integer::nhx,nhy,ihx,ihy,ivar1,ivar2
  real::gamma,smallr,smallc,gammah
  real::boxlen,boxlen2
  real::t,aexp,hexp,t2,aexp2,hexp2
  real::omega_m,omega_l,omega_k,omega_b
  real::omega_m2,omega_l2,omega_k2,omega_b2
  real::scale_l,scale_d,scale_t

  integer::ngrid,imin,imax,jmin,jmax,kmin,kmax
  integer::ncpu2,npart2,ndim2,nlevelmax2,nstep_coarse2
  integer::nx2,ny2,nz2,ngridmax2,nvarh,ndimh,nlevelmaxh
  integer::nx_full,ny_full,lmin
  integer::ix,iy,iz,ndom,impi,bit_length,maxdom
  integer,dimension(1:8)::idom,jdom,kdom,cpu_min,cpu_max
  real(KIND=8),dimension(1:8)::bounding_min,bounding_max
  real(KIND=8)::dkey,order_min,dmax
  real(KIND=8)::mtot,ddx,dxline,ddy,dex,dey,weight
  real(KIND=8)::xmin=0,xmax=1,ymin=0,ymax=1,zmin=0,zmax=1,mini=0.,maxi=0.,angl=0.
  integer::clr=1,log=1
  character(LEN=3)::filetype='mpg'
  real(KIND=8)::xxmin,xxmax,yymin,yymax,zzmin,zzmax,dx,dy
  real(KIND=4),dimension(:,:),allocatable::x,xg,var
  real(KIND=4),dimension(:)  ,allocatable::rho
  logical,dimension(:)  ,allocatable::ref
  integer,dimension(:)  ,allocatable::isp,son
  real(kind=8),dimension(1:8,1:3)::xc
  character(LEN=1)::proj='z'
  character(LEN=5)::nchar,ncharcpu,ithetachar
  character(LEN=128)::nomfich,repository,outfich,filetmp
  logical::ok,ok_part,ok_cell,do_max
  real(kind=8),dimension(:),allocatable::bound_key
  logical,dimension(:),allocatable::cpu_read
  integer,dimension(:),allocatable::cpu_list

  type level
     integer::ilevel
     integer::ngrid
     real,dimension(:,:),pointer::xg
     integer,dimension(:,:),pointer::son
     real,dimension(:,:,:),pointer::map
     integer::imin
     integer::imax
     integer::jmin
     integer::jmax
  end type level

  type(level),dimension(1:100)::grid
  
  real::theta,xidimrot,xjdimrot,xkdimrot,xidimc,xjdimc,xkdimc,dtheta
  integer::itable,ntheta,ntable,itheta,nnx,nny
  
  integer,dimension(:,:),pointer::rgb
  integer,dimension(:,:),allocatable::imr,img,imb

  interface loadct
     subroutine loadct(itable,rgb,ntable)
       implicit none
       integer,dimension(:,:),pointer::rgb
       integer::ntable,itable
     end subroutine loadct
  end interface
  
  call read_params
  itable=clr
  call loadct(itable,rgb,ntable)
  if(filetype=='mpg') then 
     ntheta=50
     dtheta=2.*3.14159265/ntheta
  else
     ntheta=0
     dtheta=0.
  end if

  !-----------------------------------------------
  ! Lecture du fichier hydro au format RAMSES
  !-----------------------------------------------
  ipos=INDEX(repository,'output_')
  nchar=repository(ipos+7:ipos+13)
  nomfich=TRIM(repository)//'/hydro_'//TRIM(nchar)//'.out00001'
  inquire(file=nomfich, exist=ok) ! verify input file 
  if ( .not. ok ) then
     print *,TRIM(nomfich)//' not found.'
     stop
  endif
  nomfich=TRIM(repository)//'/amr_'//TRIM(nchar)//'.out00001'
  inquire(file=nomfich, exist=ok) ! verify input file 
  if ( .not. ok ) then
     print *,TRIM(nomfich)//' not found.'
     stop
  endif

  nomfich=TRIM(repository)//'/amr_'//TRIM(nchar)//'.out00001'
  open(unit=10,file=nomfich,status='old',form='unformatted')
  read(10)ncpu
  read(10)ndim
  read(10)nx2,ny2,nz2
  read(10)nlevelmax
  read(10)ngridmax
  read(10)nstep_coarse
  read(10)boxlen
  read(10)t,aexp,hexp
  read(10)omega_m,omega_l,omega_k,omega_b
  read(10)scale_l,scale_d,scale_t
  close(10)
  twotondim=2**ndim

  nomfich=TRIM(repository)//'/info_'//TRIM(nchar)//'.txt'
  open(unit=10,file=nomfich,form='formatted',status='old')
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  read(10,*)
  allocate(bound_key(0:ncpu))
  allocate(cpu_read(1:ncpu))
  cpu_read=.false.
  allocate(cpu_list(1:ncpu))
  do impi=1,ncpu
     read(10,'(I8,1X,E23.15,1X,E23.15)')i,bound_key(impi-1),bound_key(impi)
  end do
  close(10)

  !-----------------------
  ! Map parameters
  !-----------------------
  if(lmax==0)then
     lmax=nlevelmax
  endif
  do_max=.false.
  if(domax==1)do_max=.true.
  write(*,*)'time=',t
  write(*,*)'Working resolution =',2**lmax
  zzmax=1.0
  zzmin=0.0
  if(ndim>2)then
  if (proj=='x')then
     idim=2
     jdim=3
     kdim=1
     xxmin=ymin ; xxmax=ymax
     yymin=zmin ; yymax=zmax
     zzmin=xmin ; zzmax=xmax
  else if (proj=='y') then
     idim=1
     jdim=3
     kdim=2
     xxmin=xmin ; xxmax=xmax
     yymin=zmin ; yymax=zmax
     zzmin=ymin ; zzmax=ymax
  else
     idim=1
     jdim=2
     kdim=3
     xxmin=xmin ; xxmax=xmax
     yymin=ymin ; yymax=ymax
     zzmin=zmin ; zzmax=zmax
  end if
  else
     idim=1
     jdim=2
     xxmin=xmin ; xxmax=xmax
     yymin=ymin ; yymax=ymax
  end if
  xidimc=(xxmin+xxmax)/2.
  xjdimc=(yymin+yymax)/2.
  xkdimc=(zzmin+zzmax)/2.


  dmax=max(xxmax-xxmin,yymax-yymin,zzmax-zzmin)
  do ilevel=1,lmax
     dx=0.5d0**ilevel
     if(dx.lt.dmax)exit
  end do
  lmin=ilevel
  bit_length=lmin-1
  maxdom=2**bit_length
  imin=0; imax=0; jmin=0; jmax=0; kmin=0; kmax=0
  if(bit_length>0)then
     imin=int(xxmin*dble(maxdom))
     imax=imin+1
     jmin=int(yymin*dble(maxdom))
     jmax=jmin+1
     kmin=int(zzmin*dble(maxdom))
     kmax=kmin+1
  endif
!  write(*,*)imin,imax,'/',maxdom
!  write(*,*)jmin,jmax,'/',maxdom
!  write(*,*)kmin,kmax,'/',maxdom

  dkey=(dble(2**(nlevelmax+1)/dble(maxdom)))**ndim
  ndom=1
  if(bit_length>0)ndom=8
  idom(1)=imin; idom(2)=imax
  idom(3)=imin; idom(4)=imax
  idom(5)=imin; idom(6)=imax
  idom(7)=imin; idom(8)=imax
  jdom(1)=jmin; jdom(2)=jmin
  jdom(3)=jmax; jdom(4)=jmax
  jdom(5)=jmin; jdom(6)=jmin
  jdom(7)=jmax; jdom(8)=jmax
  kdom(1)=kmin; kdom(2)=kmin
  kdom(3)=kmin; kdom(4)=kmin
  kdom(5)=kmax; kdom(6)=kmax
  kdom(7)=kmax; kdom(8)=kmax

  do i=1,ndom
     if(bit_length>0)then
        call hilbert3d(idom(i),jdom(i),kdom(i),order_min,bit_length,1)
     else
        order_min=0.0d0
     endif
     bounding_min(i)=(order_min)*dkey
     bounding_max(i)=(order_min+1.0D0)*dkey
!     write(*,*)i,bounding_min(i),bounding_max(i)
  end do
  cpu_min=0; cpu_max=0
  do impi=1,ncpu
     do i=1,ndom
        if (   bound_key(impi-1).le.bounding_min(i).and.&
             & bound_key(impi  ).gt.bounding_min(i))then
           cpu_min(i)=impi
        endif
        if (   bound_key(impi-1).lt.bounding_max(i).and.&
             & bound_key(impi  ).ge.bounding_max(i))then
           cpu_max(i)=impi
        endif
     end do
  end do

  ncpu_read=0
  do i=1,ndom
     do j=cpu_min(i),cpu_max(i)
        if(.not. cpu_read(j))then
           ncpu_read=ncpu_read+1
           cpu_list(ncpu_read)=j
           cpu_read(j)=.true.
        endif
     enddo
  enddo


  !-----------------------------
  ! Compute hierarchy
  !-----------------------------
  do ilevel=1,lmax
     nx_full=2**ilevel
     ny_full=2**ilevel
     imin=int(xxmin*dble(nx_full))+1
     imax=int(xxmax*dble(nx_full))+1
     jmin=int(yymin*dble(ny_full))+1
     jmax=int(yymax*dble(ny_full))+1
!     write(*,*)imin,imax
!     write(*,*)jmin,jmax
     allocate(grid(ilevel)%map(imin:imax,jmin:jmax,0:ntheta))
     grid(ilevel)%map(:,:,:)=0.0
     grid(ilevel)%imin=imin
     grid(ilevel)%imax=imax
     grid(ilevel)%jmin=jmin
     grid(ilevel)%jmax=jmax
  end do

  !-----------------------------------------------
  ! Compute projected variables
  !----------------------------------------------
  mtot=0.0d0
  do k=1,ncpu_read
     !Read amr
     icpu=cpu_list(k)
     call title(icpu,ncharcpu)
     nomfich=TRIM(repository)//'/amr_'//TRIM(nchar)//'.out'//TRIM(ncharcpu)
     open(unit=10,file=nomfich,status='old',form='unformatted')
     write(*,*)'Processing file '//TRIM(nomfich)
     read(10)ncpu2
     read(10)ndim2
     read(10)nx2,ny2,nz2
     read(10)nlevelmax2
     read(10)ngridmax2
     read(10)nstep_coarse2
     read(10)boxlen2
     read(10)t2,aexp2,hexp2
     read(10)omega_m2,omega_l2,omega_k2,omega_b2
     read(10)scale_l,scale_d,scale_t
     ! Loop over levels
     do ilevel=1,nlevelmax
        read(10)ilevela
        read(10)ngrida
        grid(ilevel)%ngrid=ngrida
!        write(*,*)ilevel,ngrida
        if(ngrida>0)then
           allocate(grid(ilevel)%xg(1:ngrida,1:ndim))
           read(10)grid(ilevel)%xg(:,1)
           read(10)grid(ilevel)%xg(:,2)
           if(ndim>2)read(10)grid(ilevel)%xg(:,3)
        end if
     end do
     do ilevel=1,nlevelmax
        ngrida=grid(ilevel)%ngrid
        if(ngrida>0)then
           allocate(grid(ilevel)%son(1:ngrida,1:twotondim))
           read(10)grid(ilevel)%son(:,1)
           do ind=1,twotondim
              read(10)grid(ilevel)%son(:,ind)
           end do
        end if
     end do
     close(10)

     !Read hydro and save leaves into cube
     nomfich=TRIM(repository)//'/hydro_'//TRIM(nchar)//'.out'//TRIM(ncharcpu)
     open(unit=11,file=nomfich,status='old',form='unformatted')
!     write(*,*)'Processing file '//TRIM(nomfich)
     read(11)ncpuh
     read(11)nvarh
     read(11)ndimh
     read(11)nlevelmaxh
     read(11)gammah
     ! Loop over levels
     do ilevel=1,nlevelmax
        dx=0.5**ilevel
        dxline=1
        if(ndim==3)dxline=dx
        nx_full=2**ilevel
        ny_full=2**ilevel
        do ind=1,twotondim
           iz=(ind-1)/4
           iy=(ind-1-4*iz)/2
           ix=(ind-1-2*iy-4*iz)
           xc(ind,1)=(dble(ix)-0.5D0)*dx
           xc(ind,2)=(dble(iy)-0.5D0)*dx
           xc(ind,3)=(dble(iz)-0.5D0)*dx
        end do
        read(11)ilevela
        read(11)ngrida
!        write(*,*)ilevel,ngrida
        if(ngrida>0)then
           allocate(x  (1:ngrida,1:ndim))
           allocate(var(1:ngrida,1:nvarh))
           allocate(rho(1:ngrida))
           allocate(ref(1:ngrida))
           do ind=1,twotondim
              do ivar=1,nvarh
                 read(11)var(:,ivar)
              end do
              do i=1,ngrida
                 x(i,1)=(grid(ilevel)%xg(i,1)+xc(ind,1))
                 x(i,2)=(grid(ilevel)%xg(i,2)+xc(ind,2))
                 if(ndim>2)x(i,3)=(grid(ilevel)%xg(i,3)+xc(ind,3))
              end do
              do i=1,ngrida
                 ref(i)=grid(ilevel)%son(i,ind)>0.and.ilevel<lmax
              end do
              select case (type)
              case (-1)
                 rho = icpu
              case (0)
                 rho = ilevel
              case (1)
                 rho = var(:,1)
              case (2)
                 rho = var(:,1)*var(:,2)
              case (3)
                 rho = var(:,1)*var(:,3)
              case (4)
                 rho = var(:,1)*var(:,4)
              case (5)
                 rho = var(:,5)
              case (6)
                 rho = var(:,1)*var(:,6)
              case (7)
                 rho = 0.5*(var(:,1)**2+var(:,2)**2+var(:,3)**2)
              end select

              if(ilevel<=lmax)then
                 do i=1,ngrida
                    ok_cell= .not.ref(i)
                    if(ok_cell)then
                       theta=angl
                       do itheta=0,ntheta
                          !rotation
                          xidimrot=(cos(theta)*(x(i,idim)-xidimc)-sin(theta)*(x(i,kdim)-xkdimc))+xidimc
                          xjdimrot=x(i,jdim)
                          xkdimrot=(sin(theta)*(x(i,idim)-xidimc)+cos(theta)*(x(i,kdim)-xkdimc))+xkdimc
                          
                          !ix=int(x(i,idim)*dble(nx_full))+1
                          !iy=int(x(i,jdim)*dble(ny_full))+1
                          ix=int(xidimrot*dble(nx_full))+1
                          iy=int(xjdimrot*dble(ny_full))+1
                          if(ndim==3)then
                             !weight=(min(x(i,kdim)+dx/2.,zzmax)-max(x(i,kdim)-dx/2.,zzmin))/dx
                             weight=(min(xkdimrot+dx/2.,zzmax)-max(xkdimrot-dx/2.,zzmin))/dx
                             weight=min(1.0d0,max(weight,0.0d0))
                          else
                             weight=1.0
                          endif
                          if(    ix>=grid(ilevel)%imin.and.&
                               & iy>=grid(ilevel)%jmin.and.&
                               & ix<=grid(ilevel)%imax.and.&
                               & iy<=grid(ilevel)%jmax)then
                             if(do_max)then
                                grid(ilevel)%map(ix,iy,itheta)=max(grid(ilevel)%map(ix,iy,itheta),rho(i))
                             else
                                grid(ilevel)%map(ix,iy,itheta)=grid(ilevel)%map(ix,iy,itheta)+rho(i)*dxline*weight/(zzmax-zzmin)
                             endif
                          endif
                          theta=theta+dtheta
                       end do
                    end if
                 end do
              endif
           end do
           deallocate(rho,x,var,ref)
        end if
     end do
     close(11)
     do ilevel=1,lmax
        ngrida=grid(ilevel)%ngrid
        if(ngrida>0)then
           grid(ilevel)%ngrid=0
           deallocate(grid(ilevel)%xg)
           deallocate(grid(ilevel)%son)
        endif
     end do

  end do

  nx_full=2**lmax
  ny_full=2**lmax
  imin=int(xxmin*dble(nx_full))+1
  imax=int(xxmax*dble(nx_full))
  jmin=int(yymin*dble(ny_full))+1
  jmax=int(yymax*dble(ny_full))

!  write(*,*)'Zoom in ',imin,imax,jmin,jmax
  do itheta=0,ntheta
     do ix=imin,imax
        xmin=((ix-0.5)/2**lmax)
        do iy=jmin,jmax
           ymin=((iy-0.5)/2**lmax)
           do ilevel=1,lmax-1
              ndom=2**ilevel
              i=int(xmin*ndom)+1
              j=int(ymin*ndom)+1
!!$           if(i<grid(ilevel)%imin.or.i>grid(ilevel)%imax.or.j<grid(ilevel)%jmin.or.j>grid(ilevel)%jmax)then
!!$              write(*,*)'OUPS'
!!$              write(*,*)ilevel
!!$              write(*,*)i,j
!!$              write(*,*)grid(ilevel)%imin,grid(ilevel)%imax
!!$              write(*,*)grid(ilevel)%jmin,grid(ilevel)%jmax
!!$              stop
!!$           endif
              if(do_max) then
                 grid(lmax)%map(ix,iy,itheta)=max(grid(lmax)%map(ix,iy,itheta), &
                      & grid(ilevel)%map(i,j,itheta))
              else
                 grid(lmax)%map(ix,iy,itheta)=grid(lmax)%map(ix,iy,itheta) + &
                      & grid(ilevel)%map(i,j,itheta)
              endif
           end do
        end do
     end do
  end do
  write(*,*)'Norm=',sum(grid(lmax)%map(imin:imax,jmin:jmax,0))/(imax-imin+1)/(jmax-jmin+1)
  
   nomfich=TRIM(outfich)
   nnx=imax-imin
   nny=jmax-jmin
   
  ! Output file
   if(filetype=='bin') then
      write(*,*)'Ecriture des donnees du fichier '//TRIM(nomfich)
      open(unit=20,file=nomfich,form='unformatted')
      write(20)imax-imin+1,jmax-jmin+1
      write(20)real(grid(lmax)%map(imin:imax,jmin:jmax,0))
      close(20)
   endif
  
   if(filetype=='mpg' .or. filetype=='mpeg' .or. filetype=='ppm') then
          write(*,*)'compute output'
          
          allocate(imr(nnx+1,nny+1))
          allocate(img(nnx+1,nny+1))
          allocate(imb(nnx+1,nny+1))
          
          call system('\rm -f rotattmp*.ppm')
          do itheta=0,ntheta
             write(*,*)'itheta=', itheta
             if (itheta==0)write(*,*)'min=',minval(grid(lmax)%map(imin:imax,jmin:jmax,itheta)),'max=',maxval(grid(lmax)%map(imin:imax,jmin:jmax,itheta))
             !Prevent to be 0 for log
             if(mini==0.)mini=sum(grid(lmax)%map(imin:imax,jmin:jmax,itheta))/size(grid(lmax)%map(imin:imax,jmin:jmax,itheta))/10.
             if(maxi==0.)maxi=maxval(grid(lmax)%map(imin:imax,jmin:jmax,itheta))
             do i=imin,imax
                do j=jmin,jmax
                   if (grid(lmax)%map(i,j,itheta)<mini) then
                      grid(lmax)%map(i,j,itheta)=mini
                   endif
                   if (grid(lmax)%map(i,j,itheta)>maxi) then
                      grid(lmax)%map(i,j,itheta)=maxi
                   endif
                end do
             end do
             if (itheta==0)write(*,*)'newmin=',mini,'newmax=',maxi
             if (log==1)grid(lmax)%map(imin:imax,jmin:jmax,itheta)=log10(grid(lmax)%map(imin:imax,jmin:jmax,itheta))
             grid(lmax)%map(imin:imax,jmin:jmax,itheta)=grid(lmax)%map(imin:imax,jmin:jmax,itheta)-minval(grid(lmax)%map(imin:imax,jmin:jmax,itheta))
             grid(lmax)%map(imin:imax,jmin:jmax,itheta)=grid(lmax)%map(imin:imax,jmin:jmax,itheta)/maxval(grid(lmax)%map(imin:imax,jmin:jmax,itheta))*(ntable-1)
             do i=imin,imax
                do j=jmin,jmax
                   imr(i+1-imin,j+1-jmin)=rgb(0,int(grid(lmax)%map(i,j,itheta)))
                   img(i+1-imin,j+1-jmin)=rgb(1,int(grid(lmax)%map(i,j,itheta)))
                   imb(i+1-imin,j+1-jmin)=rgb(2,int(grid(lmax)%map(i,j,itheta)))
                end do
             end do
             call title(itheta,ithetachar)
             filetmp='rotattmp_'//ithetachar//'.ppm' 
             call write_ppm(nnx+1,nny+1,imr,img,imb,filetmp)
          enddo
          deallocate(imr,img,imb)
          deallocate(rgb)
          deallocate(grid(lmax)%map)
          
          if(filetype=='ppm') then
             write(*,*)'write '//TRIM(nomfich)
             write(*,*)'note: you could use imagemagick to convert to your favorite picture format'
             call system('mv '//trim(filetmp)//' '//TRIM(nomfich))
          else
             !Makes movie (requires convert from imagemagick and mpeg2encode) and erases ppm 
             write(*,*) 'convert pictures ppm to movie mpeg'
             call system('convert -delay 10 rotattmp*.ppm rotattmp.mpeg')
             call system('mv rotattmp.mpeg '//TRIM(nomfich))
             call system('\rm rotattmp*.ppm')
             write(*,*)'written in '//TRIM(nomfich)
          endif
       endif
       write(*,*) 'Completed'

contains
  
  subroutine read_params
    
    implicit none
    
    integer       :: i,n
    integer       :: iargc
    character(len=4)   :: opt
    character(len=400) :: arg
    LOGICAL       :: bad, ok
    
    n = iargc()
    if (n < 4) then
       print *, 'usage: amr2mpeg   -inp  input_dir'
       print *, '                 -out  output_file'
       print *, '                 [-dir axis] '
       print *, '                 [-xmi xmin] '
       print *, '                 [-xma xmax] '
       print *, '                 [-ymi ymin] '
       print *, '                 [-yma ymax] '
       print *, '                 [-zmi zmin] '
       print *, '                 [-zma zmax] '
       print *, '                 [-lma lmax] '
       print *, '                 [-typ type] '
       print *, '                 [-pro proj] '
       print *, '                 [-clr clr ] '
       print *, '                 [-log log ] '
       print *, '                 [-min mini] '
       print *, '                 [-max maxi] '
       print *, '                 [-ang angl] '
       print *, '                 [-fil filetype] '
       print *, 'ex: amr2mpeg -inp output_00001 -out map.dat'// &
            &   ' -dir z -xmi 0.1 -xma 0.7 -lma 12'
       print *, ' '
       print *, ' type :-1 = cpu number'
       print *, ' type : 0 = ref. level (default)'
       print *, ' type : 1 = gas density'
       print *, ' type : 2 = X velocity'
       print *, ' type : 3 = Y velocity'
       print *, ' type : 4 = Z velocity'
       print *, ' type : 5 = gas pressure'
       print *, ' type : 6 = density square'
       print *, ' '
       print *, ' proj : 0 = average along line of sight (default)'
       print *, ' proj : 1 = maximum along line of sight'
       stop
    end if
    
    do i = 1,n,2
       call getarg(i,opt)
       if (i == n) then
          print '("option ",a2," has no argument")', opt
          stop 2
       end if
       call getarg(i+1,arg)
       select case (opt)
       case ('-inp')
          repository = trim(arg)
       case ('-out')
          outfich = trim(arg)
       case ('-dir')
          proj = trim(arg) 
       case ('-xmi')
          read (arg,*) xmin
       case ('-xma')
          read (arg,*) xmax
       case ('-ymi')
          read (arg,*) ymin
       case ('-yma')
          read (arg,*) ymax
       case ('-zmi')
          read (arg,*) zmin
       case ('-zma')
          read (arg,*) zmax
       case ('-lma')
          read (arg,*) lmax
       case ('-typ')
          read (arg,*) type
       case ('-pro')
          read (arg,*) domax
       case('-clr')
          read (arg,*) clr
       case('-log')
          read (arg,*) log
       case('-min')
          read (arg,*) mini
       case('-max')
          read (arg,*) maxi
       case('-ang')
          read (arg,*) angl
       case('-fil')
          read (arg,*) filetype
       case default
          print '("unknown option ",a2," ignored")', opt
       end select
    end do
    
    return
    
  end subroutine read_params
  
end program amr2mpeg

!=======================================================================
subroutine title(n,nchar)
!=======================================================================
  implicit none
  integer::n
  character*5::nchar

  character*1::nchar1
  character*2::nchar2
  character*3::nchar3
  character*4::nchar4
  character*5::nchar5

  if(n.ge.10000)then
     write(nchar5,'(i5)') n
     nchar = nchar5
  elseif(n.ge.1000)then
     write(nchar4,'(i4)') n
     nchar = '0'//nchar4
  elseif(n.ge.100)then
     write(nchar3,'(i3)') n
     nchar = '00'//nchar3
  elseif(n.ge.10)then
     write(nchar2,'(i2)') n
     nchar = '000'//nchar2
  else
     write(nchar1,'(i1)') n
     nchar = '0000'//nchar1
  endif


end subroutine title

!================================================================
!================================================================
!================================================================
!================================================================
subroutine hilbert3d(x,y,z,order,bit_length,npoint)
  implicit none

  integer     ,INTENT(IN)                     ::bit_length,npoint
  integer     ,INTENT(IN) ,dimension(1:npoint)::x,y,z
  real(kind=8),INTENT(OUT),dimension(1:npoint)::order

  logical,dimension(0:3*bit_length-1)::i_bit_mask
  logical,dimension(0:1*bit_length-1)::x_bit_mask,y_bit_mask,z_bit_mask
  integer,dimension(0:7,0:1,0:11)::state_diagram
  integer::i,ip,cstate,nstate,b0,b1,b2,sdigit,hdigit

  if(bit_length>bit_size(bit_length))then
     write(*,*)'Maximum bit length=',bit_size(bit_length)
     write(*,*)'stop in hilbert3d'
     stop
  endif

  state_diagram = RESHAPE( (/   1, 2, 3, 2, 4, 5, 3, 5,&
                            &   0, 1, 3, 2, 7, 6, 4, 5,&
                            &   2, 6, 0, 7, 8, 8, 0, 7,&
                            &   0, 7, 1, 6, 3, 4, 2, 5,&
                            &   0, 9,10, 9, 1, 1,11,11,&
                            &   0, 3, 7, 4, 1, 2, 6, 5,&
                            &   6, 0, 6,11, 9, 0, 9, 8,&
                            &   2, 3, 1, 0, 5, 4, 6, 7,&
                            &  11,11, 0, 7, 5, 9, 0, 7,&
                            &   4, 3, 5, 2, 7, 0, 6, 1,&
                            &   4, 4, 8, 8, 0, 6,10, 6,&
                            &   6, 5, 1, 2, 7, 4, 0, 3,&
                            &   5, 7, 5, 3, 1, 1,11,11,&
                            &   4, 7, 3, 0, 5, 6, 2, 1,&
                            &   6, 1, 6,10, 9, 4, 9,10,&
                            &   6, 7, 5, 4, 1, 0, 2, 3,&
                            &  10, 3, 1, 1,10, 3, 5, 9,&
                            &   2, 5, 3, 4, 1, 6, 0, 7,&
                            &   4, 4, 8, 8, 2, 7, 2, 3,&
                            &   2, 1, 5, 6, 3, 0, 4, 7,&
                            &   7, 2,11, 2, 7, 5, 8, 5,&
                            &   4, 5, 7, 6, 3, 2, 0, 1,&
                            &  10, 3, 2, 6,10, 3, 4, 4,&
                            &   6, 1, 7, 0, 5, 2, 4, 3 /), &
                            & (/8 ,2, 12 /) )

  do ip=1,npoint

     ! convert to binary
     do i=0,bit_length-1
        x_bit_mask(i)=btest(x(ip),i)
        y_bit_mask(i)=btest(y(ip),i)
        z_bit_mask(i)=btest(z(ip),i)
     enddo

     ! interleave bits
     do i=0,bit_length-1
        i_bit_mask(3*i+2)=x_bit_mask(i)
        i_bit_mask(3*i+1)=y_bit_mask(i)
        i_bit_mask(3*i  )=z_bit_mask(i)
     end do

     ! build Hilbert ordering using state diagram
     cstate=0
     do i=bit_length-1,0,-1
        b2=0 ; if(i_bit_mask(3*i+2))b2=1
        b1=0 ; if(i_bit_mask(3*i+1))b1=1
        b0=0 ; if(i_bit_mask(3*i  ))b0=1
        sdigit=b2*4+b1*2+b0
        nstate=state_diagram(sdigit,0,cstate)
        hdigit=state_diagram(sdigit,1,cstate)
        i_bit_mask(3*i+2)=btest(hdigit,2)
        i_bit_mask(3*i+1)=btest(hdigit,1)
        i_bit_mask(3*i  )=btest(hdigit,0)
        cstate=nstate
     enddo

     ! save Hilbert key as double precision real
     order(ip)=0.
     do i=0,3*bit_length-1
        b0=0 ; if(i_bit_mask(i))b0=1
        order(ip)=order(ip)+dble(b0)*dble(2)**i
     end do

  end do

end subroutine hilbert3d
!================================================================
!================================================================
!================================================================
!================================================================

subroutine loadct(itable,rgb,ntable)
  !Load color table
  !A color table of size ntable is a 3*ntable array rgb which associates to the ntable subscripts a triplet R G B.
  !Simples one are ntable=256, rgb(0:2,i)=i from 0 to 255 grey levels or rgb(0,i)=i red levels 
  !For example I created a rainbow table with more levels by going from black to white on the edges of the r,g,b cube.
  !Black-blue-cyan-green-yellow-red-purple-white
  integer,dimension(:,:),pointer::rgb
  integer::i,itot,itable,ntable
  
  !leger doublon a la jonction a corriger...
  !IDL-like color tables
  
  select case(itable)
     
  case(-1)
     !reverse grey levels
     write(*,*)'white-black color table'
     ntable=256
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=255-i
        rgb(1,itot)=255-i
        rgb(2,itot)=255-i
        itot=itot+1
     end do
     
  case(0)
     !grey levels
     write(*,*)'black-white color table'
     ntable=256
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=i
        rgb(1,itot)=i
        rgb(2,itot)=i
        itot=itot+1
     end do
     
  case(1)
     !blue-white table
     write(*,*)'blue-white color table'
     ntable=512
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=0
        rgb(1,itot)=0
        rgb(2,itot)=i
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=i
        rgb(1,itot)=i
        rgb(2,itot)=255
        itot=itot+1
     end do

  case(3)
     !red-white table
     write(*,*)'red-white color table'
     ntable=512
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=i
        rgb(1,itot)=0
        rgb(2,itot)=0
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=255
        rgb(1,itot)=i
        rgb(2,itot)=i
        itot=itot+1
     end do
     

  case(13)
     !rainbow
     write(*,*)'rainbow color table'
     ntable=1792
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=0
        rgb(1,itot)=0
        rgb(2,itot)=i
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=0
        rgb(1,itot)=i
        rgb(2,itot)=255
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=0
        rgb(1,itot)=255
        rgb(2,itot)=255-i
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=i
        rgb(1,itot)=255
        rgb(2,itot)=0
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=255
        rgb(1,itot)=255-i
        rgb(2,itot)=0
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=255
        rgb(1,itot)=0
        rgb(2,itot)=i
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=255
        rgb(1,itot)=i
        rgb(2,itot)=255
        itot=itot+1
     end do
     
  case(33)
     !blue-red
     ntable=768
     write(*,*)'blue-red color table'
     allocate(rgb(0:2,0:ntable-1))
     itot=0
     do i=0,255
        rgb(0,itot)=0
        rgb(1,itot)=i
        rgb(2,itot)=255
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=i
        rgb(1,itot)=255
        rgb(2,itot)=255-i
        itot=itot+1
     end do
     do i=0,255
        rgb(0,itot)=255
        rgb(1,itot)=255-i
        rgb(2,itot)=0
        itot=itot+1
     end do
  end select
  
end subroutine loadct

subroutine write_ppm(nx,ny,imr,img,imb,file)
!This routine write a .PPM file de type P3 to be converted with imagemagick 
!and erased afterwards since it's very memory consuming. It is however very
!convenient format because very straightforward to write.
!File size and write time could be improved with PPM P6 (check google).
!Yann Rasera March 8th 2007
!
!Format PPM3 (the file is written in ASCII)
!P3
!nx ny
!255 (maxvalue)
!R G B
!R G B
!R G B
!....
!nx*ny lines with the same order as you read.
!That's it!!!

integer::nx,ny
integer,dimension(nx,ny)::imr,img,imb !has to be between 0 and 255
character(LEN=128)::file
integer::i,j

open(unit=10,file=trim(file),form='formatted')
write(10,'(a2)')'P3'
write(10,'(I4,1x,I4)')nx,ny
write(10,'(a3)')'255'
do j=ny,1,-1 
   do i=1,nx
      write(10,'(I3,1x,I3,1x,I3)')imr(i,j),img(i,j),imb(i,j)
   end do
end do
close(10)
end subroutine write_ppm

